@@ -1,24 +1,24 @@
 
 --- Module implementing the LuaRocks "lint" command.
 -- Utility function that checks syntax of the rockspec.
-module("luarocks.lint", package.seeall)
+local lint = {}
+package.loaded["luarocks.lint"] = lint
 
 local util = require("luarocks.util")
 local download = require("luarocks.download")
 local fetch = require("luarocks.fetch")
 
-help_summary = "Check syntax of a rockspec."
-help_arguments = "<rockspec>"
-help = [[
+util.add_run_function(lint)
+lint.help_summary = "Check syntax of a rockspec."
+lint.help_arguments = "<rockspec>"
+lint.help = [[
 This is a utility function that checks the syntax of a rockspec.
 
 It returns success or failure if the text of a rockspec is
 syntactically correct.
 ]]
 
-function run(...)
-   local flags, input = util.parse_flags(...)
-   
+function lint.command(flags, input)
    if not input then
       return nil, "Argument missing. "..util.see_help("lint")
    end
@@ -51,3 +51,5 @@ function run(...)
 
    return ok, ok or filename.." failed consistency checks."
 end
+
+return lint
