@@ -1,20 +1,30 @@
 
--- Create rock files, packing sources or binaries.
+--- Module implementing the LuaRocks "pack" command.
+-- Creates a rock, packing sources or binaries.
 local pack = {}
+package.loaded["luarocks.pack"] = pack
 
 local unpack = unpack or table.unpack
 
-local queries = require("luarocks.queries")
 local path = require("luarocks.path")
 local repos = require("luarocks.repos")
 local fetch = require("luarocks.fetch")
 local fs = require("luarocks.fs")
-local cfg = require("luarocks.core.cfg")
+local cfg = require("luarocks.cfg")
 local util = require("luarocks.util")
 local dir = require("luarocks.dir")
 local manif = require("luarocks.manif")
 local search = require("luarocks.search")
-local signing = require("luarocks.signing")
+
+util.add_run_function(pack)
+pack.help_summary = "Create a rock, packing sources or binaries."
+pack.help_arguments = "{<rockspec>|<name> [<version>]}"
+pack.help = [[
+Argument may be a rockspec file, for creating a source rock,
+or the name of an installed package, for creating a binary rock.
+In the latter case, the app version may be given as a second
+argument.
+]]
 
 --- Create a source rock.
 -- Packages a rockspec and its required source files in a rock
@@ -30,7 +40,7 @@ function pack.pack_source_rock(rockspec_file)
    if err then
       return nil, "Error loading rockspec: "..err
    end
-   rockspec_file = rockspec.local_abs_filename
+   rockspec_file = rockspec.local_filename
 
    local name_version = rockspec.name .. "-" .. rockspec.version
    local rock_file = fs.absolute_name(name_version .. ".src.rock")
@@ -43,10 +53,9 @@ function pack.pack_source_rock(rockspec_file)
    if not ok then return nil, err end
 
    fs.delete(rock_file)
-   fs.copy(rockspec_file, source_dir, "read")
-   ok, err = fs.zip(rock_file, dir.base_name(rockspec_file), dir.base_name(source_file))
-   if not ok then
-      return nil, "Failed packing "..rock_file.." - "..err
+   fs.copy(rockspec_file, source_dir, cfg.perm_read)
+   if not fs.zip(rock_file, dir.base_name(rockspec_file), dir.base_name(source_file)) then
+      return nil, "Failed packing "..rock_file
    end
    fs.pop_dir()
 
@@ -79,21 +88,26 @@ end
 -- @param tree string or nil: An optional tree to pick the package from.
 -- @return string or (nil, string): The filename of the resulting
 -- .src.rock file; or nil and an error message.
-function pack.pack_installed_rock(query, tree)
+local function do_pack_binary_rock(name, version, tree)
+   assert(type(name) == "string")
+   assert(type(version) == "string" or not version)
 
-   local name, version, repo, repo_url = search.pick_installed_rock(query, tree)
+   local repo, repo_url
+   name, version, repo, repo_url = search.pick_installed_rock(name, version, tree)
    if not name then
       return nil, version
    end
 
-   local root = path.root_from_rocks_dir(repo_url)
+   local root = path.root_dir(repo_url)
    local prefix = path.install_dir(name, version, root)
    if not fs.exists(prefix) then
       return nil, "'"..name.." "..version.."' does not seem to be an installed rock."
    end
    
-   local rock_manifest, err = manif.load_rock_manifest(name, version, root)
-   if not rock_manifest then return nil, err end
+   local rock_manifest = manif.load_rock_manifest(name, version, root)
+   if not rock_manifest then
+      return nil, "rock_manifest file not found for "..name.." "..version.." - not a LuaRocks 2 tree?"
+   end
 
    local name_version = name .. "-" .. version
    local rock_file = fs.absolute_name(name_version .. "."..cfg.arch..".rock")
@@ -103,12 +117,12 @@ function pack.pack_installed_rock(query, tree)
 
    local is_binary = false
    if rock_manifest.lib then
-      local ok, err = copy_back_files(name, version, rock_manifest.lib, path.deploy_lib_dir(repo), dir.path(temp_dir, "lib"), "exec")
+      local ok, err = copy_back_files(name, version, rock_manifest.lib, path.deploy_lib_dir(root), dir.path(temp_dir, "lib"), cfg.perm_exec)
       if not ok then return nil, "Failed copying back files: " .. err end
       is_binary = true
    end
    if rock_manifest.lua then
-      local ok, err = copy_back_files(name, version, rock_manifest.lua, path.deploy_lua_dir(repo), dir.path(temp_dir, "lua"), "read")
+      local ok, err = copy_back_files(name, version, rock_manifest.lua, path.deploy_lua_dir(root), dir.path(temp_dir, "lua"), cfg.perm_read)
       if not ok then return nil, "Failed copying back files: " .. err end
    end
    
@@ -126,26 +140,7 @@ function pack.pack_installed_rock(query, tree)
    return rock_file
 end
 
-function pack.report_and_sign_local_file(file, err, sign)
-   if err then
-      return nil, err
-   end
-   local sigfile
-   if sign then
-      sigfile, err = signing.sign_file(file)
-      util.printout()
-   end
-   util.printout("Packed: "..file)
-   if sigfile then
-      util.printout("Signature stored in: "..sigfile)
-   end
-   if err then
-      return nil, err
-   end
-   return true
-end
-
-function pack.pack_binary_rock(name, namespace, version, sign, cmd)
+function pack.pack_binary_rock(name, version, cmd, ...)
 
    -- The --pack-binary-rock option for "luarocks build" basically performs
    -- "luarocks build" on a temporary tree and then "luarocks pack". The
@@ -161,7 +156,7 @@ function pack.pack_binary_rock(name, namespace, version, sign, cmd)
    util.schedule_function(fs.delete, temp_dir)
 
    path.use_tree(temp_dir)
-   local ok, err = cmd()
+   local ok, err = cmd(...)
    if not ok then
       return nil, err
    end
@@ -169,9 +164,34 @@ function pack.pack_binary_rock(name, namespace, version, sign, cmd)
    if not rname then
       rname, rversion = name, version
    end
-   local query = queries.new(rname, namespace, rversion)
-   local file, err = pack.pack_installed_rock(query, temp_dir)
-   return pack.report_and_sign_local_file(file, err, sign)
+   return do_pack_binary_rock(rname, rversion, temp_dir)
+end
+
+--- Driver function for the "pack" command.
+-- @param arg string:  may be a rockspec file, for creating a source rock,
+-- or the name of an installed package, for creating a binary rock.
+-- @param version string or nil: if the name of a package is given, a
+-- version may also be passed.
+-- @return boolean or (nil, string): true if successful or nil followed
+-- by an error message.
+function pack.command(flags, arg, version)
+   assert(type(version) == "string" or not version)
+   if type(arg) ~= "string" then
+      return nil, "Argument missing. "..util.see_help("pack")
+   end
+
+   local file, err
+   if arg:match(".*%.rockspec") then
+      file, err = pack.pack_source_rock(arg)
+   else
+      file, err = do_pack_binary_rock(arg, version, flags["tree"])
+   end
+   if err then
+      return nil, err
+   else
+      util.printout("Packed: "..file)
+      return true
+   end
 end
 
 return pack
